@@ -1,22 +1,26 @@
-# 
+#
 # This file is part of Dist-Zilla-Plugin-Git
-# 
+#
 # This software is copyright (c) 2009 by Jerome Quelin.
-# 
+#
 # This is free software; you can redistribute it and/or modify it under
 # the same terms as the Perl 5 programming language system itself.
-# 
+#
 use 5.008;
 use strict;
 use warnings;
 
 package Dist::Zilla::Plugin::Git::CommitBuild;
 BEGIN {
-  $Dist::Zilla::Plugin::Git::CommitBuild::VERSION = '1.101330';
+  $Dist::Zilla::Plugin::Git::CommitBuild::VERSION = '1.101650';
 }
 # ABSTRACT: checkin build results on separate branch
 
 use Git::Wrapper;
+use Git;
+use File::chdir;
+use File::Spec::Functions qw/ rel2abs catfile /;
+use File::Temp;
 use Moose;
 use MooseX::Has::Sugar;
 use MooseX::Types::Moose qw{ Str };
@@ -48,31 +52,59 @@ has message => ( ro, isa => Str, default => 'Build results of %h (on %b)', requi
 # -- role implementation
 
 sub after_build {
-	my $self  = shift;
-	my $args  = shift;
-	my $src_dir = abs_path('.');
+    my ( $self, $args) = @_;
 
-	my $src   = Git::Wrapper->new($src_dir);
-	my $target_branch = _format_branch( $self->branch, $src );
+    my $repo    = Git->repository;
+    my $tmp_dir = File::Temp->newdir( CLEANUP => 1) ;
+    my $src     = Git::Wrapper->new('.');
 
-	my $exists = eval { $src->rev_parse( '--verify', '-q', $target_branch ); 1; };
+    my $dir = rel2abs( $args->{build_root} );
 
-	eval {
-		my $build = Git::Wrapper->new( $args->{build_root} );
-		$build->init('-q');
-		$build->remote('add','src',$src_dir);
-		$build->fetch(qw(-q src));
-		if($exists){
-			$build->reset('--soft', "src/$target_branch");
-		}
-		$build->add('.');
-		$build->commit('-a', -m => _format_message($self->message, $src));
-		$build->checkout('-b',$target_branch);
-		$build->push('src', $target_branch);
-	};
-	if (my $e = $@) {
-		$self->log_fatal("failed to commit build: $e");
-	}
+    my $tree = do {
+        # don't overwrite the user's index
+        local $ENV{GIT_INDEX_FILE} = catfile( $tmp_dir, "temp_git_index" );
+        local $ENV{GIT_DIR}        = catfile( $CWD,     '.git' );
+        local $ENV{GIT_WORK_TREE}  = $dir;
+
+        local $CWD = $dir;
+
+        my $write_tree_repo = Git->repository;
+
+        $write_tree_repo->command(qw(add -v --force .));
+        $write_tree_repo->command_oneline("write-tree");
+    };
+
+    my $target_branch = _format_branch( $self->branch, $src );
+
+    # no change, abort
+    return
+      if eval {
+              $repo->command_oneline( 'rev-parse', '-q', '--verify',
+                  $target_branch );
+        }
+          and not $repo->command( qw/ diff --stat /, $target_branch, $tree );
+
+    my @parents = grep {
+        eval { $repo->command_oneline( 'rev-parse', '-q', '--verify', $_ ) }
+    } $target_branch, 'HEAD';
+
+    my ( $pid, $in, $out, $ctx ) =
+      Git::command_bidi_pipe( "commit-tree", $tree,
+        map { ( -p => $_ ) } @parents,
+      );
+
+    $out->print( _format_message( $self->message, $src ) );
+
+    close $out;
+    my $buf = '';
+    open $out, '<', \$buf;
+
+    chomp( my $commit = <$in> );
+
+    Git::command_close_bidi_pipe( $pid, $in, $out, $ctx );
+
+    $repo->command_noisy( 'update-ref', 'refs/heads/' . $target_branch,
+        $commit );
 }
 
 1;
@@ -86,7 +118,7 @@ Dist::Zilla::Plugin::Git::CommitBuild - checkin build results on separate branch
 
 =head1 VERSION
 
-version 1.101330
+version 1.101650
 
 =head1 SYNOPSIS
 
@@ -103,7 +135,7 @@ Once the build is done, this plugin will commit the results of the
 build to a branch that is completely separate from your regular code
 branches (i.e. with a different root commit).  This potentially makes
 your repository more useful to those who may not have L<Dist::Zilla>
-and all of it's dependencies installed.
+and all of its dependencies installed.
 
 The plugin accepts the following options:
 
